/**
 * This class contains all the necessary functions to deal with the UI.
 * Its init() function should be called immediately upon successful project load.
 */
function UiController() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var mainId = "main";
    var popupId = "popup";
    var popupContentId = "popupcontent";
    var popupButtonId = "popupbutton";
    var currentTheme = "defaulttheme";
    var themeTimeout = null; // Current timeout-set function for clearing theme changes.

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Creates the header of a tooltip.
     * @param {string} header The header's contents.
     * @returns {string} The header.
     */
    function createToolTipHeader(header) {
        if (header) {
            return `<div class="tooltipheader">${header}</div>`;
        }
        return "";
    }

    /**
     * Creates the body of a tooltip.
     * @param {string} body The body's contents.
     * @returns {string} The vody.
     */
    function createToolTipBody(body) {
        if (body) {
            return `<div class="tooltipbody">${body}</div>`;
        }
        return "";
    }

    /**
     * Creates the footer of a tooltip.
     * @param {string} footer The footer's contents.
     * @returns {string} The footer.
     */
    function createToolTipFooter(footer) {
        if (footer) {
            return `<div class="tooltipbottom">${footer}</div>`;
        }
        return "";
    }

    /**
     *  Creates and returns a tooltip.
     *  @param {string} body The body of the tooltip. Required.
     *  @param {string} header The header of the tooltip. Optional.
     *  @param {string} footer The element that when moused over should show the tooltip. Optional.
     *  @param {string} style Additional style to add to the tooltip. Optional.
     *  @return {string} The tooltip.
     */
    function createToolTip(body, header, footer, style) {
        var div = `<div class="tooltip" style="${style ? style : ""}">`;
        div += createToolTipHeader(header);
        div += createToolTipBody(body);
        div += createToolTipFooter(footer);
        div += `</div>`;
        return div;
    }

    /**
     * Registers all required listeners.
     */
    function registerListeners() {
        $("#" + popupButtonId).click(that.hideFullscreenPopup);
        global.settings.hideNsfwListener.register(hideNsfwIfRequired);
        global.settings.textSizeListener.register(setFontSize);
    }

    /**
     * Clears the temporary class used to facilitate theme changes.
     * Automatically cleared at the end of a theme change, or when cancelThemeChange is called.
     */
    function clearThemeChangeClass() {
        $("body, body *").removeClass("themetransition");
    }

    /**
     * Hides NSFW images if the settings demand it.
     */
    function hideNsfwIfRequired() {
        var cssId = "UiController-HideNsfw";
        $("#" + cssId).remove();
        if (global.settings.getHideNsfw()) {
            var sheet = document.createElement('style');
            sheet.innerHTML = `img.nsfw { display: none; }`;
            sheet.id = cssId;
            document.head.appendChild(sheet);
        }
    }

    /**
     * Sets the font size of body text.
     */
    function setFontSize() {
        var cssId = "UiController-FontSize";
        $("#" + cssId).remove();
        var size = global.settings.getTextSize();
        var sheet = document.createElement('style');
        sheet.innerHTML = `.text { font-size: ${size}px; }`;
        sheet.id = cssId;
        document.head.appendChild(sheet);
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.genericHeaderId = "content";
    this.genericFooterId = "finisher";

    // Icon creator.
    this.icons = new Icons();

    // The collection of tabs.
    this.tabs = new AllTabs();
    
    // The collection of tabs.
    this.muteButton = new GlobalMuteButton();

    // The scroll level of the current scene.
    this.sceneScroll = 0;

    // Load bar.
    this.loadbar = new LoadBar();

    // Popup menu.
    this.menu = new PopupMenu();

    // All throbbers (i.e. loading icons).
    this.throbbers = new AllThrobbers();

    // Change notification builder.
    this.change = new ChangeNotificationBuilder();

    // Image background enabler.
    this.imageBackground = new ImageBackground();

    // Image in text enabler.
    this.inlineImage = new ImageInText();

    // Generates themes dynamically to avoid having to include a billion CSS files.
    this.themeLoader = new ThemeLoader(true);

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Initializes the UI controller.
     */
    this.init = function () {
        hideNsfwIfRequired();
        setFontSize();
        that.tabs.init();
        that.muteButton.init();
        that.menu.init();
        that.themeLoader.init();
        registerListeners();
    };

    /**
     * Clears the main view, and loads the passed string into it as HTML.
     * @param {string} toLoad The HTML to load.
     */
    this.loadToMain = function (toLoad) {
        var main = $("#" + mainId);
        main.empty();
        main.html(toLoad);
    };

    /**
     * Appends the passed HTML string to the main view.
     * If the main content is wrapped in a container div, pass its id to this function.
     * @param {string} toAppend The HTML to append.
     * @param {string} id Optional: the id of the object to append the HTML to.
     */
    this.appendToMain = function (toAppend, id) {
        var element;
        if (id) {
            element = $("#" + id);
        } else {
            element = $("#" + mainId);
        }
        element.append(toAppend);
    };

    /**
     * Appends the passed HTML string to the main view, before a standard finisher.
     * Usage of this function is preferred over usage of appendToMain.
     * @param {string} toAppend The HTML to append.
     */
    this.appendBeforeFinisher = function (toAppend) {
        var element = $("#" + that.genericFooterId);
        element.before(toAppend);
    };

    /**
     * Builds a generic content container and returns it as a HTML string.
     * The id of this generic content container can be queried via the 
     * genericHeaderId member of this UiController. Recommended to be used as
     * the start of every piece of content.
     * @returns {string} A generic content container as a HTML string.
     */
    this.buildGenericHeader = function () {
        return '<div class="content" id="' + that.genericHeaderId + '">';
    };

    /**
     * Builds a footer to match the generic content container header and returns
     * it as a HTML string. Recommended to be used as the end of every piece of 
     * content.
     * @returns {string} A generic content container footer as a HTML string.
     */
    this.buildGenericFooter = function () {
        return '<div class="finish" id="' + that.genericFooterId + '"></div></div>';
    };

    /**
     * Turns the passed element into a tooltip container.
     * @param {string} mouseOver The element that when moused over should show the tooltip.
     * @param {string} body The body of the tooltip. Required.
     * @param {string} header The header of the tooltip. Optional.
     * @param {string} footer The footer of the tooltip. Optional.
     * @param {string} style Additional style to add to the tooltip. Optional.
     * @returns {string} The tooltipcontainer containing the tooltip.
     */
    this.makeToolTip = function (mouseOver, body, header, footer, style) {
        return `<div class="tooltipcontainer">${createToolTip(body, header, footer, style)}${mouseOver}</div>`;
    };

    /**
     * Creates a static tooltip (that will always be revealed) and returns it.
     * @param {string} body The body of the tooltip. Required.
     * @param {string} header The header of the tooltip. Optional.
     * @param {string} footer The element that when moused over should show the tooltip. Optional.
     * @param {string} style Additional style to add to the tooltip. Optional.
     * @returns {string} The tooltip.
     */
    this.makeStaticToolTip = function (body, header, footer, style) {
        return `${createToolTip(body, header, footer, style)}`;
    };

    /**
     * Fades out all content in the main container.
     * @param {function} fun The function to execute after fading out content.
     */
    this.fadeOutContent = function (fun) {
        //$("#" + mainId).children().fadeOut(400, fun); // Calls fun once for every element hidden.
        $("#" + mainId).children().fadeOut(400).promise().done(fun); // Calls fun once.
    };

    /**
     * Sets the scroll level of the main div to the current sceneScroll.
     */
    this.setScroll = function () {
        $("#" + mainId).scrollTop(that.sceneScroll);
    };

    /**
     * Scrolls the main div to the bottom.
     */
    this.scrollBottom = function () {
        that.sceneScroll = document.getElementById(mainId).scrollHeight;
        that.setScroll();
    };

    /**
     * Sets the scroll level of the main div to 0.
     */
    this.resetScroll = function () {
        that.sceneScroll = 0;
        that.setScroll();
    };

    /**
     * Fades in a fullscreen popup with a single button which will hide the popup
     * once more.
     * @param {string} content The content of the popup window.
     */
    this.showFullscreenPopup = function (content) {
        var popupContent = $("#" + popupContentId);
        popupContent.html(content);
        popupContent.append('<div class="popupfinish"></div>');
        popupContent.append('<div class="popupfinish"></div>');
        $("#" + popupId).fadeIn(400);
        setTimeout(function () { // Firefox fix
            $("#popupcenter").scrollTop(0);
        }, 5);
    };

    /**
     * Applies the passed theme. If no theme is passed, the default theme is applied.
     * Does nothing if the passed theme is already applied.
     * Invalidates previous theme changes.
     * @param {string} theme The name of the theme to apply.
     */
    this.applyTheme = function (theme) {
        theme = global.settings.getThemesDisabled() ? false : theme;
        theme = theme ? theme : "defaulttheme";
        if (currentTheme === theme) {
            return;
        }
        that.cancelThemeChangeTransition();
        currentTheme = theme;
        $("body, body *").addClass("themetransition");
        $(".throbber").removeClass("themetransition");
        themeTimeout = setTimeout(clearThemeChangeClass, 2000);
        $("body").attr('class', 'themetransition');
        $("body").addClass(theme);
        that.themeLoader.load(theme);
    };

    /**
     * Cancels theme change transition and clears up the temporary CSS-class used to facilitate it.
     * In essence: does immediately what applyTheme would have done in two seconds.
     */
    this.cancelThemeChangeTransition = function () {
        clearTimeout(themeTimeout);
        themeTimeout = null;
        clearThemeChangeClass();
    };

    /**
     * Fades out a fullscreen popup.
     */
    this.hideFullscreenPopup = function () {
        $("#" + popupId).fadeOut(400);
    };

    /**
     * Slowly fades in the fullscreen loading screen.
     * Should ideally never be used.
     */
    this.showLoadingScreen = function () {
        $("#loadingscreen").fadeIn(800);
    };

    /**
     * Slowly fades out the fullscreen loading screen.
     * Should ideally only be used during startup.
     */
    this.hideLoadingScreen = function () {
        $("#loadingscreen").fadeOut(800);
    };

    /**
     * Returns a BB-code enabled conditional fail if the settings approve of it.
     * Returns the empty string otherwise.
     * @param {string} fail The failure message.
     * @returns {string} The failure koBBold code or an empty string, depending on the settings.
     */
    this.condFail = function (fail) {
        if (global.settings.getConditionsInvisible()) {
            return "";
        }
        return `[fail]${fail}[/fail]`;
    };

    /**
     * Returns a BB-code enabled conditional success if the settings approve of it.
     * Returns the empty string otherwise.
     * @param {string} succ The success message.
     * @returns {string} The success koBBold code or an empty string, depending on the settings.
     */
    this.condSucc = function (succ) {
        if (global.settings.getConditionsInvisible()) {
            return "";
        }
        return `[succ]${succ}[/succ]`;
    };
}
