/**
 * Class capable of generating a merchant's inventory screen and registering listeners for it.
 * @param {string} uniqueId Unique id of this shop, to allow for multiple shops in a single scene.
 * @param {array} items The items the merchant has for sale as an array of ItemAmount objects.
 * @param {float} sellmod The value-modifier at which the kobold sells items.
 * @param {float} buymod The value-modifier at which the kobold buys items.
 * @param {int} coins How many coins the kobold has at scene-load time.
 * @param {boolean} disabled Whether or not this shop is still active.
 * @param {function} sellListener A listener for when the kobold sells something from its inventory. Gets passed the item which was sold. Has no return value.
 * @param {function} buyListener A listener for when the kobold buys something from the merchant. 
 *                               Gets passed the index in the item-array of the item which was bought.
 */
function MerchantInventoryGenerator(uniqueId, items, coins, sellmod, buymod, disabled, sellListener, buyListener) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var coinSpanId = uniqueId + "Coins";
    var buyButtonClass = "itemrowuse";
    
    var coinSound = global.audio.load("res/audio/ui/coin.mp3", 80);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    
    function playSoundAndCallSellListener(item) {
        global.audio.play(coinSound);
        sellListener(item);
    }

    /**
     * Sets all required global variables.
     */
    function setGlobals() {
        global.kobold.inventory.enterSafeZone();
        global.kobold.inventory.enterSellZone();
        global.kobold.inventory.sellMod = sellmod;
        global.kobold.inventory.sellListener = playSoundAndCallSellListener;
    }

    /**
     * Resets all required global variables.
     */
    function resetGlobals() {
        global.kobold.inventory.exitSafeZone();
        global.kobold.inventory.exitSellZone();
        global.kobold.inventory.sellMod = 1.0;
        global.kobold.inventory.sellListener = null;
    }

    /**
     * Updates amount and fades bought item away if required, 
     * then updates all of the buttons and the amount of coins.
     * @param {int} id Index of the bought item in the items array.
     */
    function updateEverything(id) {
        var itemAmount = items[id];
        $("#" + uniqueId + id).find(".itemrownumber").html(itemAmount.amount);
        if (itemAmount.amount <= 0) {
            $("#" + uniqueId + id).hide(400);
        }
        $("#" + coinSpanId).html(coins);
        for (var i = 0; i < items.length; i++) {
            var item = items[i].item;
            if (Math.ceil(item.price() * buymod) > coins) {
                $("#" + uniqueId + i).find("button").prop("disabled", true);
            }
        }
    }

    function bought(id) {
        items[id].amount--;
        var item = items[id].item;
        coins -= Math.ceil(item.price() * buymod);
        global.kobold.inventory.coins -= Math.ceil(item.price() * buymod);
        global.kobold.inventory.addItem(item);
        updateEverything(id);
    }

    /**
     * Builds and returns the number section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The number section of an item row.
     */
    function buildItemRowNumber(itemAmount) {
        return `<div class="itemrownumber">${itemAmount.amount}</div>`;
    }

    /**
     * Builds and returns the X section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The X section of an item row.
     */
    function buildItemRowX(itemAmount) {
        return `<div class="itemrowx">x</div>`;
    }

    /**
     * Builds and returns the name section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The name section of an item row.
     */
    function buildItemRowName(itemAmount) {
        return `<div class="itemrowname">${global.kobold.inventory.createItemWithTooltip(itemAmount.item, true)}</div>`;
    }

    /**
     * Builds and returns the value section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The value section of an item row.
     */
    function buildItemRowValue(itemAmount) {
        return `<div class="itemrowvalue">${Math.ceil(itemAmount.item.price() * buymod)} ${global.ui.icons.coin()}</div>`;
    }

    /**
     * Builds and returns the sell section of an item row.
     * This is just an empty container to preserve proper spacing.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The sell section of an item row.
     */
    function buildItemRowSell(itemAmount) {
        var toReturn = `<div class="itemrowsellcontainer">`;
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the buy section of an item row.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @returns {string} The use section of an item row.
     */
    function buildItemRowBuy(itemAmount) {
        var canBuy = coins >= Math.ceil(itemAmount.item.price() * buymod);
        var toReturn = `<div class="itemrowusecontainer">`;
        toReturn += `<button class="button ${buyButtonClass}" ${disabled || !canBuy ? "disabled" : ""}>Buy</button>`;
        toReturn += `</div>`;
        return toReturn;
    }

    /**
     * Builds and returns the innards of a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the kobold has.
     * @returns {string} The innards of a row for a single item.
     */
    function buildItemRowInnards(itemAmount) {
        var innards = buildItemRowNumber(itemAmount);
        innards += buildItemRowX(itemAmount);
        innards += buildItemRowName(itemAmount);
        innards += buildItemRowValue(itemAmount);
        innards += buildItemRowSell(itemAmount);
        innards += buildItemRowBuy(itemAmount);
        return innards;
    }

    /**
     * Builds and returns a row for a single item.
     * @argument {ItemAmount} itemAmount The item and how much of it the merchant has.
     * @argument {int} i The 0-based row number of the item.
     * @returns {string} A row for a single item.
     */
    function buildItemRow(itemAmount, i) {
        var id = uniqueId + i;
        var rowClass = "itemroweven";
        if (i % 2) {
            rowClass = "itemrowodd";
        }
        var row = `<div id="${id}" class="itemrow ${rowClass}">`;
        row += buildItemRowInnards(itemAmount);
        return row + `</div>`;
    }

    /**
     * Disables all of the buy buttons of this store.
     */
    function disableButtons() {
        $("#" + uniqueId).find("button").prop("disabled", true);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns the merchant's inventory.
     * @returns {string} The bag section of the inventory, as HTML string.
     */
    this.build = function () {
        if (!disabled) {
            setGlobals();
        }
        var bag = `<div id="${uniqueId}" class="shop">`;
        bag += `<div class="text">You have <span id="${coinSpanId}">${coins}</span> coin${coins !== 1 ? "s" : ""}.</div>`;
        // Check if all zero, then also no items
        if (items.length < 1) {
            bag += `<div class="text">The merchant has no items.</div>`;
        } else {
            for (var i = 0; i < items.length; i++) {
                if (items[i].amount <= 0) {
                    continue;
                }
                bag += buildItemRow(items[i], i);
            }
        }
        bag += `</div>`;
        return bag;
    };

    /**
     * Registers listeners for the buy buttons of this store.
     */
    this.registerListeners = function () {
        if (disabled) {
            return;
        }
        $("#" + uniqueId).find("." + buyButtonClass).click(function () {
            var button = $(this);
            var row = $(button.parent().parent()[0]);
            var messyId = row.attr("id");
            var id = messyId.slice(uniqueId.length);
            var itemAmount = items[id];            
            global.audio.play(coinSound);
            bought(id, itemAmount);
            buyListener(id);
        });
    };

    /**
     * Disables this store.
     */
    this.disable = function () {
        disabled = true;
        resetGlobals();
        disableButtons();
    };
}

