/**
 * This class acts as a dedicated controller for the introduction of the Forest scene.
 * @param {string} sceneId The id of the scene.
 */
function ForestIntroductionController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = sceneId;

    var thinkingChoice = "introRec";
    var recoveringStamina = "introRecStamina";
    var recoveringChestCoveredId = "introChest";
    var itemChoice = "introItem";
    var itemSharedId = "introItemShared";
    var itemChoiceStamina = "itemChoiceStamina";
    var itemChoiceBaseGender = "itemChoiceBaseGender";
    var pathChoice = "introPath";
    var pathChoiceStamina = "pathChoiceStamina";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Adds equipment to the kobold depending on the chosen item.
     * @param {GenderEnum} choice The selected item.
     */
    function itemChoiceEquipment(choice) {
        if(choice === 0) {
            return;
        }
        global.kobold.inventory.equipment.wear(choice);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the thinking choice.
     * @param {ForestIntroductionThinkingChoiceEnum} choice The selected item.
     */
    this.makeThinkingChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[thinkingChoice] = choice;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the thinking choice from history and returns it.
     * May return undefined if the choice has not yet been made.
     */
    this.getThinkingChoice = function () {
        var history = global.kobold.history.get(id);
        return history[thinkingChoice];
    };

    /**
     * Returns true if the thinking choice has already been made. False otherwise.
     */
    this.hasMadeThinkingChoice = function () {
        return global.kobold.history.has(id, thinkingChoice);
    };

    /**
     * Makes the item choice, giving the kobold a weapon.
     * @param {Item} choice The selected item.
     */
    this.makeItemChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[itemChoice] = choice;
        global.kobold.history.set(id, history);
        itemChoiceEquipment(choice);
    };

    /**
     * Fetches the item choice from history and returns it.
     * May return undefined if the choice has not yet been made.
     */
    this.getItemChoice = function () {
        var history = global.kobold.history.get(id);
        return history[itemChoice];
    };

    /**
     * Returns true if the item choice has already been made. False otherwise.
     */
    this.hasMadeItemChoice = function () {
        return global.kobold.history.has(id, itemChoice);
    };
    
    /**
     * Makes the item shared changes.
     */
    this.itemShared = function () {
        global.kobold.history.doIfNotSet(id, itemSharedId, true, function () {
            if(that.getThinkingChoice() === ForestIntroductionThinkingChoiceEnum.PRESENT) {
                global.kobold.history.shared.caravanBandits.knowledge = CaravanBanditsKnowledgeEnum.SUSPICIOUS;
            }
        });
    };

    /**
     * Makes the path choice.
     * @param {ForestIntroductionPathChoiceEnum} choice The selected item.
     */
    this.makePathChoice = function (choice) {
        var history = global.kobold.history.get(id);
        history[pathChoice] = choice;
        global.kobold.history.set(id, history);
    };

    /**
     * Fetches the path choice from history and returns it.
     * May return undefined if the choice has not yet been made.
     */
    this.getPathChoice = function () {
        var history = global.kobold.history.get(id);
        return history[pathChoice];
    };

    /**
     * Returns true if the path choice has already been made. False otherwise.
     */
    this.hasMadePathChoice = function () {
        return global.kobold.history.has(id, pathChoice);
    };

    /**
     * Reduces the kobold's stamina and increases its sanity if required.
     * @returns {string} HTML describing the change.
     */
    this.recoveringStatusChange = function () {
        if (!global.kobold.history.has(id, recoveringStamina)) {
            global.kobold.status.stamina.decrease(60, "Running for your life");
            global.kobold.status.mental.decrease(10, "Close to passing out");
            global.kobold.status.physical.decrease(10, "Overexerted");
            global.kobold.inventory.coins += 203;
            var history = global.kobold.history.get(id);
            history[recoveringStamina] = true;
            global.kobold.history.set(id, history);
        }
        return global.ui.change.buildNotification([
            new Change("Overexerted", "Health", -10),
            new Change("Close to passing out", "Sanity", -10),
            new Change("Out of breath", "Stamina", -60)
        ]);
    };

    /**
     * Returns whether or not the kobold's chest was covered at the recovery-phase.
     * @returns {string} Whether or not the kobold's chest was covered at the recovery-phase.
     */
    this.recoveringChestCovered = function () {
        if (!global.kobold.history.has(id, recoveringChestCoveredId)) {
            var history = global.kobold.history.get(id);
            history[recoveringChestCoveredId] = !global.kobold.inventory.equipment.chestExposed();
            global.kobold.history.set(id, history);
        }
        return global.kobold.history.get(id)[recoveringChestCoveredId];
    };
    
    /**
     * Returns the kobold's base gender at the item choice phase.
     * @returns {GenderEnum} The kobold's base gender at the item choice phase.
     */
    this.itemChoiceBaseGEnder = function () {
        if (!global.kobold.history.has(id, itemChoiceBaseGender)) {
            var history = global.kobold.history.get(id);
            history[itemChoiceBaseGender] = global.kobold.body.baseGender();
            global.kobold.history.set(id, history);
        }
        return global.kobold.history.get(id)[itemChoiceBaseGender];
    };

    /**
     * Increase the kobold's stamina if required.
     * @returns {string} HTML describing the change.
     */
    this.itemSelectionStatusChange = function () {
        if (!global.kobold.history.has(id, itemChoiceStamina)) {
            global.kobold.status.stamina.increase(10, "Recovering");
            global.kobold.status.mental.increase(10, "Calmed down");
            global.kobold.status.physical.increase(5, "Recovering");
            var history = global.kobold.history.get(id);
            history[itemChoiceStamina] = true;
            global.kobold.history.set(id, history);
        }
        ``
        return global.ui.change.buildNotification([
            new Change("Recovering", "Health", +5), 
            new Change("Calmed down", "Sanity", +10), 
            new Change("Recovering", "Stamina", +10)
        ]);
    };

    /**
     * Increase the kobold's stamina if required.
     * @returns {string} HTML describing the change.
     */
    this.pathSelectionStatusChange = function () {
        if (!global.kobold.history.has(id, pathChoiceStamina)) {
            global.kobold.status.stamina.increase(10, "Recovering");
            global.kobold.status.mental.increase(5, "Coin-laden optimism");
            global.kobold.status.physical.increase(5, "Recovering");
            var history = global.kobold.history.get(id);
            history[pathChoiceStamina] = true;
            global.kobold.history.set(id, history);
        }
        return global.ui.change.buildNotification([
            new Change("Recovering", "Health", +5), 
            new Change("Cautious optimism", "Sanity", +5), 
            new Change("Recovering", "Stamina", +10)
        ]);
    };

}
